﻿using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows;
using System.Windows.Interop;
using System.Windows.Media;

namespace GlassEffect
{
    /// <summary>
    /// Clase estática para el manejo del DWM
    /// </summary>
    public static class DWMHelper
    {
        /// <summary>
        /// API para generar el efecto Glass
        /// </summary>
        /// <param name="hwnd">Puntero a la ventana</param>
        /// <param name="pMarInset">Márgenes nuevos</param>
        [DllImport("DwmApi.dll")]
        private static extern int DwmExtendFrameIntoClientArea(IntPtr hwnd, ref Margins pMarInset);

        /// <summary>
        /// Estructura con los márgenes nuevos
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        private struct Margins
        {
            public int cxLeftWidth;
            public int cxRightWidth;
            public int cyTopHeight;
            public int cyBottomHeight;
        }

        /// <summary>
        /// Aplica el efecto Glass a una ventana
        /// </summary>
        /// <param name="win">Ventana</param>
        /// <param name="left">Cantida de pixeles de la izquierda</param>
        /// <param name="right">Cantida de pixeles de la derecha</param>
        /// <param name="top">Cantida de pixeles de arriba</param>
        /// <param name="bottom">Cantida de pixeles de abajo</param>
        public static void ApplyGlass(Window win, int left, int right, int top, int bottom)
        {
            //Obtengo el puntero Win32 a la ventana
            WindowInteropHelper windowInterop = new WindowInteropHelper(win);
            IntPtr windowHandle = windowInterop.Handle;
            HwndSource.FromHwnd(windowHandle).CompositionTarget.BackgroundColor = Colors.Transparent;

            //Obtengo los DPIs del sistema operativo
            Graphics g = Graphics.FromHwnd(windowHandle);

            //Establezco los márgenes
            Margins margins = new Margins();
            margins.cxLeftWidth = Convert.ToInt32(left * (g.DpiX / 96));
            margins.cxRightWidth = Convert.ToInt32(right * (g.DpiX / 96));
            margins.cyTopHeight = Convert.ToInt32(top * (g.DpiY / 96));
            margins.cyBottomHeight = Convert.ToInt32(bottom * (g.DpiY / 96));

            // Extend the glass frame.
            if (DwmExtendFrameIntoClientArea(windowHandle, ref margins) < 0)
                throw new NotSupportedException("Error en la operación");
        }
    }
}